#ifndef VELOCITYEXTFROMVELOCITYFIELD_H
#define VELOCITYEXTFROMVELOCITYFIELD_H

#include "AdaptInfo.h"
#include "BasisFunction.h"
#include "DOFVector.h"
#include "ElInfo.h"
// #include "MemoryManager.h"

#include "NormEps.h"
#include "VelocityExt.h"

using namespace AMDiS;


/* #ifndef NORMEPS_H */
/* #define NORMEPS_H */
/* ///////////////////////////////////////////////////////////////////////////// */
/* //  c l a s s   N o r m E p s                                              // */
/* ///////////////////////////////////////////////////////////////////////////// */
/* class NormEps */
/* { */
/*  public: */
/*   MEMORY_MANAGED(NormEps); */

/*   /\** */
/*    * Calculates regularized norm of WorldVector x. */
/*    *\/ */
/*   static double calcNormEps(const WorldVector<double> &x)  */
/*   { */
/*     FUNCNAME("NormEps::calcNormEps()"); */

/*     static double result; */

/*     TEST_EXIT(eps > 1.e-15)("illegal eps for norm regularization !\n"); */
/*     result = x*x + eps*eps; */

/*     return(std::sqrt(result)); */
/*   }; */

/*   /\** */
/*    * Calculates square of regularized norm of WorldVector x. */
/*    *\/ */
/*   static double calcNormEps2(const WorldVector<double> &x)  */
/*   { */
/*     FUNCNAME("NormEps::calcNormEps2()"); */

/*     static double result; */

/*     TEST_EXIT(eps > 1.e-15)("illegal eps for norm regularization !\n"); */
/*     result = x*x + eps*eps; */

/*     return(result); */
/*   }; */

/*   /\** */
/*    * Set regularization epsilon eps. */
/*    *\/ */
/*   static inline void setEps() { */
/*     FUNCNAME("NormEps::setEps()"); */

/*     eps = 0.0; */
/*     GET_PARAMETER(0, "NormEps->epsilon for norm regularization", "%f", &eps); */

/*     TEST_EXIT(eps > 1.e-15) */
/*       ("illegal eps for norm regularization !\n"); */
/*   }; */

/*  protected: */
/*   /\** */
/*    * Epsilon for regularization. */
/*    *\/ */
/*   static double eps; */
/* }; */
/* #endif  // NORMEPS_H */

/////////////////////////////////////////////////////////////////////////////
//  c l a s s   V e l o c i t y E x t F r o m V e l o c i t y F i e l d    //
/////////////////////////////////////////////////////////////////////////////
class VelocityExtFromVelocityField : public VelocityExt
{
 public:
  MEMORY_MANAGED(VelocityExtFromVelocityField);

  /**
   * Constructor.
   */
  VelocityExtFromVelocityField(int dim_)
    : VelocityExt(dim_),
    lSFct(NULL),
    elNormalVel(dim_, NO_INIT),
    basFcts(NULL)
  {
    FUNCNAME("VelocityExtFromVelocityField::VelocityExtFromVelocityField()");

    lSFctVal = GET_MEMORY(double, dim+1);

    // ===== set epsilon for norm regularization =====
    NormEps::setEps();
  };

  /**
   * Destructor.
   */
  ~VelocityExtFromVelocityField()
  {
    FREE_MEMORY(lSFctVal, double, dim+1);
  };

  /**
   * Set velocity field.
   */
  void setVelocityField(std::vector<DOFVector<double> *> &velField_,
			const DOFVector<double> *lSFct_,
			DOFVector<double> *velDOF_)
  {
    FUNCNAME("VelocityExtFromVelocityField::setVelocityField()");

    nVelDOFs = 1;

    velField = velField_;
    lSFct = lSFct_;
    velDOF.clear();
    velDOF.push_back(velDOF_);
    origVelDOF.clear();

    TEST_EXIT(lSFct)("level set function not defined !\n");
    TEST_EXIT((int)velField.size() == dim)("illegal velocity field !\n");
    TEST_EXIT(lSFct->getFESpace() == velDOF_->getFESpace())
      ("different feSpaces !\n");

    basFcts = lSFct->getFESpace()->getBasisFcts();
  };

  /**
   * Adaption of VelocityExt::calcVelocityBoundary().
   * Used to initialize normal velocity at interface elements.
   */
  void calcVelocityBoundary(DegreeOfFreedom *locInd, const int indexV);

  /**
   * Sets elInfo.
   */
  inline void setElInfo(ElInfo *elInfo_) {
    elInfo = elInfo_;
  };

 protected:
  /**
   * Velocity field at interface.
   */
  std::vector<DOFVector<double> *> velField;

  /**
   * Interface is zero level set of level set function lSFct.
   */
  const DOFVector<double> *lSFct;

  /**
   * Normal velocity on a single element. Used to set normal velocity
   * in velDOF on interface elements.
   */
  DimVec<double> elNormalVel;

  /**
   * Values of level set function in vertices of element.
   */
  double *lSFctVal;

  /**
   * Basis functions.
   */
  const BasisFunction *basFcts;

  /**
   * ElInfo used in calcVelocityBoundary().
   */
  ElInfo *elInfo;
};

#endif  // VELOCITYEXTFROMVELOCITYFIELD_H
